import { useCallback } from 'react';
import { nanoid } from 'nanoid';
import { useImageUploads } from './useImageUploads';
import { useFileUploads } from './useFileUploads';
import { checkUploadPermissions } from './utils';
import { isLocalAttachment, isLocalImageAttachment, isUploadedImage } from '../../Attachment';
import { useChannelActionContext, useChannelStateContext, useChatContext, useTranslationContext, } from '../../../context';
const apiMaxNumberOfFiles = 10;
const ensureIsLocalAttachment = (attachment) => {
    if (isLocalAttachment(attachment)) {
        return attachment;
    }
    const { localMetadata, ...rest } = attachment;
    return {
        localMetadata: {
            ...(localMetadata ?? {}),
            id: localMetadata?.id || nanoid(),
        },
        ...rest,
    };
};
export const useAttachments = (props, state, dispatch, textareaRef) => {
    const { doFileUploadRequest, doImageUploadRequest, errorHandler, noFiles } = props;
    const { fileUploads, imageUploads } = state;
    const { getAppSettings } = useChatContext('useAttachments');
    const { t } = useTranslationContext('useAttachments');
    const { addNotification } = useChannelActionContext('useAttachments');
    const { channel, maxNumberOfFiles, multipleUploads } = useChannelStateContext('useAttachments');
    const { removeFile, uploadFile } = useFileUploads(props, state, dispatch);
    const { removeImage, uploadImage } = useImageUploads(props, state, dispatch);
    // Number of files that the user can still add. Should never be more than the amount allowed by the API.
    // If multipleUploads is false, we only want to allow a single upload.
    const maxFilesAllowed = !multipleUploads ? 1 : maxNumberOfFiles || apiMaxNumberOfFiles;
    // OG attachments should not be counted towards "numberOfImages"
    const numberOfImages = Object.values(imageUploads).filter(({ og_scrape_url, state }) => state !== 'failed' && !og_scrape_url).length;
    const numberOfFiles = Object.values(fileUploads).filter(({ state }) => state !== 'failed').length;
    const numberOfUploads = numberOfImages + numberOfFiles;
    const maxFilesLeft = maxFilesAllowed - numberOfUploads;
    const uploadNewFiles = useCallback((files) => {
        Array.from(files)
            .slice(0, maxFilesLeft)
            .forEach((file) => {
            const id = nanoid();
            if (file.type.startsWith('image/') &&
                !file.type.endsWith('.photoshop') // photoshop files begin with 'image/'
            ) {
                dispatch({
                    file,
                    id,
                    previewUri: URL.createObjectURL?.(file),
                    state: 'uploading',
                    type: 'setImageUpload',
                });
            }
            else if (file instanceof File && !noFiles) {
                dispatch({ file, id, state: 'uploading', type: 'setFileUpload' });
            }
        });
        textareaRef?.current?.focus();
    }, 
    // eslint-disable-next-line react-hooks/exhaustive-deps
    [maxFilesLeft, noFiles]);
    const removeAttachments = useCallback((ids) => {
        if (!ids.length)
            return;
        dispatch({ ids, type: 'removeAttachments' });
    }, [dispatch]);
    const upsertAttachments = useCallback((attachments) => {
        if (!attachments.length)
            return;
        dispatch({
            attachments: attachments.map(ensureIsLocalAttachment),
            type: 'upsertAttachments',
        });
    }, [dispatch]);
    const uploadAttachment = useCallback(async (att) => {
        const { localMetadata, ...attachment } = att;
        if (!localMetadata?.file)
            return att;
        const isImage = isUploadedImage(attachment);
        const id = localMetadata?.id ?? nanoid();
        const { file } = localMetadata;
        const canUpload = await checkUploadPermissions({
            addNotification,
            file,
            getAppSettings,
            t,
            uploadType: isImage ? 'image' : 'file',
        });
        if (!canUpload) {
            const notificationText = t('Missing permissions to upload the attachment');
            console.error(new Error(notificationText));
            addNotification(notificationText, 'error');
            return att;
        }
        upsertAttachments([
            {
                ...attachment,
                localMetadata: {
                    ...localMetadata,
                    id,
                    uploadState: 'uploading',
                },
            },
        ]);
        let response;
        try {
            const doUploadRequest = isImage ? doImageUploadRequest : doFileUploadRequest;
            if (doUploadRequest) {
                response = await doUploadRequest(file, channel);
            }
            else {
                response = await channel[isImage ? 'sendImage' : 'sendFile'](file);
            }
        }
        catch (error) {
            let finalError = { message: t('Error uploading attachment'), name: 'Error' };
            if (typeof error.message === 'string') {
                finalError = error;
            }
            else if (typeof error === 'object') {
                finalError = Object.assign(finalError, error);
            }
            console.error(finalError);
            addNotification(finalError.message, 'error');
            const failedAttachment = {
                ...attachment,
                localMetadata: {
                    ...localMetadata,
                    uploadState: 'failed',
                },
            };
            upsertAttachments([failedAttachment]);
            if (errorHandler) {
                errorHandler(finalError, 'upload-attachment', file);
            }
            return failedAttachment;
        }
        if (!response) {
            // Copied this from useImageUpload / useFileUpload. Not sure how failure could be handled on app level.
            // If doUploadRequest returns any falsy value, then don't create the upload preview.
            // This is for the case if someone wants to handle failure on app level.
            removeAttachments([id]);
            return;
        }
        const uploadedAttachment = {
            ...attachment,
            localMetadata: {
                ...localMetadata,
                uploadState: 'finished',
            },
        };
        if (isLocalImageAttachment(uploadedAttachment)) {
            if (uploadedAttachment.localMetadata.previewUri) {
                URL.revokeObjectURL(uploadedAttachment.localMetadata.previewUri);
                delete uploadedAttachment.localMetadata.previewUri;
            }
            uploadedAttachment.image_url = response.file;
        }
        else {
            uploadedAttachment.asset_url = response.file;
        }
        upsertAttachments([uploadedAttachment]);
        return uploadedAttachment;
    }, [
        addNotification,
        channel,
        doFileUploadRequest,
        doImageUploadRequest,
        errorHandler,
        getAppSettings,
        removeAttachments,
        t,
        upsertAttachments,
    ]);
    return {
        maxFilesLeft,
        numberOfUploads,
        removeAttachments,
        removeFile,
        removeImage,
        uploadAttachment,
        uploadFile,
        uploadImage,
        uploadNewFiles,
        upsertAttachments,
    };
};
